from ddc.task_build_docker_image import __build_docker_image
from ddc.utils import stream_exec_cmd


def start_build_assets(cwd: str, output_dir: str, docker_image_tag: str = None):
    all_output = []
    exit_code = -1
    image_tag = None
    try:
        if not docker_image_tag:
            exit_code, image_tag = __build_docker_image(all_output, cwd)
        else:
            image_tag = docker_image_tag

        docker_args = [
            "-v {output_dir}:/tmp/ddc_build_mount".format(output_dir=output_dir),
        ]

        all_output.append("Copy assets")
        exit_code, mix_output = stream_exec_cmd(
            "docker run --rm {docker_args} {image_tag} cp -R /usr/app_assets /tmp/ddc_build_mount".format(
                image_tag=image_tag, docker_args=" ".join(docker_args),
            )
        )
        all_output.append(mix_output)
    except StopIteration:
        pass
    finally:
        if not docker_image_tag and image_tag:
            stream_exec_cmd("docker image rm " + image_tag)

    return exit_code, "\n".join(all_output)



import pathlib
import pathspec
import logging
import platform
import subprocess
import sys
from os.path import expanduser
import os
import time
import hashlib


def __get_os():
    if os.name == "nt":
        return "windows"

    if platform.system() == "Darwin":
        return "macos"

    return "linux"


OS_NAME = __get_os()
DDC_DIR = "/.ddc/"


def __build_path(path):
    if OS_NAME == "windows":
        path = path.replace("/", "\\")
    full_path = expanduser("~") + path
    return full_path


def read_ddc_file(path) -> str:
    """
    :param path:  example: "developer_settings.json"
    :return: dict
    """
    ret = None
    full_path = __build_path(DDC_DIR + path)
    if os.path.isfile(full_path):
        with open(full_path, "r") as myfile:
            ret = myfile.read()
    return ret


def write_ddc_file(path, value: str) -> None:
    """
    :param path:  example: "developer_settings.json"
    :param value: dict
    """
    ddc_dir = __build_path(DDC_DIR)
    os.makedirs(ddc_dir, exist_ok=True)
    full_path = __build_path(DDC_DIR + path)
    with open(full_path, "w") as myfile:
        myfile.write(value)


def get_path_ddc_file(path) -> str:
    """
    :param path:  example: "developer_settings.json"
    """
    return __build_path(DDC_DIR + path)


def exec_cmd(cmd, print_exec_cmd=True):
    if print_exec_cmd:
        logging.info("Run cmd: " + cmd)
    return subprocess.getstatusoutput(cmd)


def stream_exec_cmd(cmd: str, print_exec_cmd=True):
    if print_exec_cmd:
        logging.info("Run cmd: " + str(cmd))

    # если надо будет печатать на экран в момент выполенения, то подменить subprocess.PIPE на что-то свое
    proc = subprocess.Popen(
        cmd,
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE,
        shell=True,
        universal_newlines=True,
    )
    (out, err) = proc.communicate()
    return proc.returncode, out + "\n" + err


def run_if_time_has_passed(lock_key: str, ttl_min: int, callback_fn):
    """Run callback function is lock time is passed"""
    lock_file = "fce_" + lock_key + ".loc"
    current_ts = int(time.time())
    saved_ts = read_ddc_file(lock_file)
    if not saved_ts:
        saved_ts = 0
    else:
        saved_ts = int(saved_ts)
    if current_ts - saved_ts > ttl_min * 60:
        callback_fn()
        write_ddc_file(lock_file, str(current_ts))


def query_yes_no(question, default="yes"):
    """Ask a yes/no question via raw_input() and return their answer.

    "question" is a string that is presented to the user.
    "default" is the presumed answer if the user just hits <Enter>.
        It must be "yes" (the default), "no" or None (meaning
        an answer is required of the user).

    The "answer" return value is True for "yes" or False for "no".
    """
    valid = {"yes": True, "y": True, "ye": True, "no": False, "n": False}
    if default is None:
        prompt = " [y/n] "
    elif default == "yes":
        prompt = " [Y/n] "
    elif default == "no":
        prompt = " [y/N] "
    else:
        raise ValueError("invalid default answer: '%s'" % default)

    while True:
        sys.stdout.write(question + prompt)
        choice = input().lower()
        if default is not None and choice == "":
            return valid[default]
        elif choice in valid:
            return valid[choice]
        else:
            sys.stdout.write("Please respond with 'yes' or 'no' " "(or 'y' or 'n').\n")


def iterate_source_files_with_gitignore(base_path: str):
    """
    Iterate all files in base_path including hidden dir and files.
    Using .gitignore for skip items.
    """
    gitignores = []
    # glob.glob skip hidden files. We can't use it.
    for fileref in pathlib.Path(base_path).glob("**/*.*"):
        file = str(fileref)
        if file.endswith(".gitignore"):
            with open(file, "r") as f:
                spec_lines = []
                for line in f.readlines():
                    line = line.strip()
                    if line.endswith("*"):
                        line = line.rstrip("*")
                    spec_lines.append(line)

            spec = pathspec.PathSpec.from_lines("gitwildmatch", spec_lines)
            git_ignore_dir = file[0 : -len(".gitignore")]
            gitignores.append(
                {"dir": git_ignore_dir, "dir_len": len(git_ignore_dir), "spec": spec}
            )
            continue

        is_ignored_file = False
        for git_ignore_info in gitignores:
            if file.startswith(git_ignore_info["dir"]):
                cutted_file = file[git_ignore_info["dir_len"] :]
                is_ignored_file = git_ignore_info["spec"].match_file(cutted_file)
                if is_ignored_file:
                    break
        if is_ignored_file:
            continue
        yield file


def get_exists_langs(base_path: str):
    """
    Compute which languages use in base_path
    """
    extensions = {
        "js": False,
        "ts": False,
        "java": False,
        "php": False,
        "py": False,
        "go": False,
    }
    for file in iterate_source_files_with_gitignore(base_path):
        for f_ext, is_found in extensions.items():
            if not is_found:
                if file.endswith("." + f_ext):
                    extensions[f_ext] = True

    langs = []
    for ext_name, ext_exists in extensions.items():
        if ext_exists:
            langs.append(ext_name)
    return langs


def get_lang_tests_dir(base_path: str):
    """
    Get list of folders with langs for run tests
    """
    dirs = {}

    def discover_py_php(file: str):
        # folder with tests MUST be named as "/tests"
        if "/tests" not in file:
            return

        parts = file.split("/")
        parts_cnt = len(parts)
        if parts_cnt < 2:
            return

        dirname = parts[parts_cnt - 2]
        if dirname != "tests":
            return

        filename = parts[parts_cnt - 1]
        dir_ = "/".join(parts[0:-2])

        if filename == "bootstrap.php":
            dirs[dir_] = {"lang": "php", "dir_for_run_tests": dir_}

        if filename == "__init__.py":
            dirs[dir_] = {"lang": "py", "dir_for_run_tests": dir_}

        if filename.startswith("test_") and filename.endswith(".py"):
            dirs[dir_] = {"lang": "py", "dir_for_run_tests": dir_}

    def discover_java(file: str):
        # folder with tests MUST be named as "/src/test/java"
        if "/src/test/java" not in file:
            return
        if not file.endswith(".java"):
            return
        # find root java module
        parts = file.split("/src/test/java")
        if len(parts) > 0:
            dir_ = parts[0]
            dirs[dir_] = {"lang": "java", "dir_for_run_tests": dir_}

    for file in iterate_source_files_with_gitignore(base_path):
        discover_py_php(file)
        discover_java(file)

    return list(dirs.values())


def md5(input_str: str):
    return hashlib.md5(input_str.encode("utf-8")).hexdigest()



from __future__ import absolute_import

import logging
import os

import fire

from ddc.task_black import start_black
from ddc.task_build_docker_image import start_build_docker_image
from ddc.task_lint import start_lint
from ddc.task_meta import start_meta
from ddc.info import (
    print_about_version,
    check_pip_version,
    upgrade_pip_version_to_latest,
)
from ddc.task_test import start_test
from ddc.utils import query_yes_no, run_if_time_has_passed


def print_and_exit(exit_code, out):
    sout = out.strip()
    if len(sout) > 0:
        print(sout)  # noqa
    if exit_code == 0:
        print("👌 Looks good!")  # noqa
    else:
        print("⛔️ We have error")  # noqa
    exit(exit_code)


class Cli(object):
    """
    Devision Developers Cli Lib.
    👉 Press 'q' for exit
    """

    def __init__(self) -> None:
        super().__init__()
        self.cwd = os.getcwd()

    def meta(self, workdir=None):
        """
        Start Meta App in dev mode
        :param workdir: str. By default is current dir
        :return:
        """
        if not workdir:
            workdir = self.cwd
        logging.info("Starter Meta. Work dir: " + workdir)
        start_meta(self.cwd)

    def lint(self, lang="py", workdir=None):
        """
        Start lint of current directory. Run in a directory that needs linting
        :param workdir: str. By default is current dir
        :param lang: str Can be: [ py | php ]
        :return:
        """
        if not workdir:
            workdir = self.cwd
        logging.info("Start linter. Work dir: " + workdir)
        exit_code, out = start_lint(workdir, lang)
        print_and_exit(exit_code, out)

    def black(self, lang="py", workdir=None):
        """
        The uncompromising code formatter. Run in a directory that needs formatting
        :param workdir: str. By default is current dir
        :param lang: str Can be: [ py ]
        :return:
        """
        if not workdir:
            workdir = self.cwd
        logging.info("Start black. Work dir: " + workdir)
        exit_code, out = start_black(workdir, lang)
        print_and_exit(exit_code, out)

    def build_docker_image(self, workdir=None):
        """
        Build docker image
        :param workdir: str. By default is current dir
        :return:
        """
        if not workdir:
            workdir = self.cwd
        logging.info("Start build docker image. Work dir: " + workdir)
        exit_code, out, image_tag = start_build_docker_image(workdir)
        if not exit_code:
            print("Image tag: " + str(image_tag))  # noqa
        print_and_exit(exit_code, out)

    def test(self, lang="py", workdir=None, save_test_results=False):
        """
        Test file MUST be in directory /tests in workdir
        If tests are not found, they are considered to have failed.
        :param workdir: str. By default is current dir
        :param lang: str Can be: [ py ]
        :param save_test_results: bool Save results in junit format to directory "test-results"
        :return:
        """
        if not workdir:
            workdir = self.cwd
        logging.info("Start tests. Work dir: " + workdir)
        exit_code, out = start_test(workdir, lang, save_test_results)
        print_and_exit(exit_code, out)

    def version(self):
        """Show the ddc version information"""
        print_about_version()

    def upgrade(self):
        """Upgrade cli lib"""
        upgrade_pip_version_to_latest()


def try_check_version():
    if not check_pip_version():
        print("⚡️ You use not latest version of ddc")  # noqa
        if query_yes_no("Do you want update lib?"):
            upgrade_pip_version_to_latest()


def main():
    run_if_time_has_passed("pip_version", 60, try_check_version)
    fire.Fire(Cli, name="ddc")


if __name__ == "__main__":
    main()



from ddc.utils import stream_exec_cmd, run_if_time_has_passed


def start_black(cwd: str, lang: str):
    image_tag = "apisgarpun/pronto-black-{lang}".format(lang=lang)

    def _pull():
        stream_exec_cmd("docker pull {image_tag}".format(image_tag=image_tag))

    run_if_time_has_passed("black-" + lang, 10, _pull)
    return stream_exec_cmd(
        "docker run --rm -v {cwd}:/usr/app {image_tag}".format(
            cwd=cwd, image_tag=image_tag
        )
    )



from ddc.utils import stream_exec_cmd, run_if_time_has_passed, get_exists_langs


def _run_lint(cwd, lang):
    image_tag = "apisgarpun/pronto-linter-{lang}".format(lang=lang)

    def _pull():
        stream_exec_cmd("docker pull {image_tag}".format(image_tag=image_tag))

    run_if_time_has_passed("lint-" + lang, 10, _pull)
    exit_code, max_output = stream_exec_cmd(
        "docker run --rm -v {cwd}:/usr/app {image_tag}".format(
            cwd=cwd, image_tag=image_tag
        )
    )
    if (
        image_tag + ":latest not found" in max_output
        or image_tag + ", repository does not exist" in max_output
    ):
        exit_code = 0
        max_output = "Skip check lang. We not found pronto linter plugin. lang: " + lang

    new_output = []
    cwd_prefix = cwd if cwd.endswith("/") else cwd + "/"
    for line in max_output.split("\n"):
        if str(line).startswith("./"):
            line = cwd_prefix + line[2:]
        new_output.append(line)
    return exit_code, "\n".join(new_output)


def start_lint(cwd, lang):
    if lang == "auto":
        new_output = []
        exit_code = 0
        for lang in get_exists_langs(cwd):
            new_output.append("Check lang: " + lang)
            exit_code, lang_output = _run_lint(cwd, lang)
            new_output.append(lang_output)
            if exit_code == 0:
                new_output.append("OK")
            else:
                new_output.append("Failure...")
                break
        return exit_code, "\n".join(new_output)
    return _run_lint(cwd, lang)
